use std::fs::File;
use std::io::prelude::*;

use axum::{
    http::StatusCode,
    routing::{get, post},
    Json, Router,
};
use serde::{Deserialize, Serialize};
use spaceapi::Status;

mod status;

// TODO: Figure out where to ackshully put this.
const STATUS_FILE: &str = "status.json";

#[tokio::main]
async fn main() {
    // initialize tracing
    // tracing_subscriber::fmt::init();

    // build our application with a route
    let app = Router::new()
        .route("/status.json", get(root))
        .route("/api/update.php", post(the_doors));

    // run our app with hyper, listening globally on port 3000
    let listener = tokio::net::TcpListener::bind("0.0.0.0:3000").await.unwrap();
    axum::serve(listener, app).await.unwrap();
}

// Just output the current file. It may cease to exist.
async fn root() -> String {
    // TODO: handle error
    std::fs::read_to_string(STATUS_FILE).unwrap_or(String::from("KAPOTT"))
}

// Input type for the API: Both fields are optional.
#[derive(Serialize, Deserialize)]
struct TheDoors {
    aerie: Option<bool>,
    cellar: Option<bool>,
}

// The door can see through your soul.
// https://www.youtube.com/watch?v=bDQDp00oTP4
async fn the_doors(Json(payload): Json<TheDoors>) -> StatusCode {
    // TODO: handle error
    let contents = std::fs::read_to_string(STATUS_FILE).expect("FCKAFD");
    let status: Status = serde_json::from_str(&contents).unwrap();

    if let Some(sens) = status.sensors {
        // Get the current status as read from the file.
        let mut doors = sens.door_locked.into_iter();
        let aerie_current = doors
            .find(|d| d.metadata.location == "aerie")
            .unwrap_or(status::get_aerie(false));
        let cellar_current = doors
            .find(|d| d.metadata.location == "cellar")
            .unwrap_or(status::get_cellar(false));

        // New door status: fall back to respective current value if no update
        // is provided.
        let aerie = match payload.aerie {
            Some(s) => s,
            None => aerie_current.value,
        };
        let cellar = match payload.cellar {
            Some(s) => s,
            None => cellar_current.value,
        };

        let new_doors = status::TheDoors { aerie, cellar };
        let (sensors, state) = status::update(new_doors);

        let s = status::status(sensors, state);
        let s = serde_json::to_string(&s).unwrap();

        let mut file = File::create(STATUS_FILE).unwrap();
        file.write_all(s.as_bytes()).unwrap();
    }

    StatusCode::CREATED
}
